unit TtfComp;

interface

uses
  Windows, Messages, SysUtils, Classes,
  Graphics, Controls, Forms, Dialogs;

type
  TTtfError=(TtfFileNoError,TtfFileInvalid,TtfFileNoAccess,TtfUnknownError);
type
  TTtfComp = class(TComponent)
  private
    { Private declarations }
    FTtfError : TTtfError;
    FPlatformID :Word;
    FPSpecEncID :Word;
    FLanguageID :Word;
    FCopyright     :string;
    FFontFamilyName:string;
    FFontSubFamName:string;
    FFontFullName  :string;
    FFontVersion   :string;
    FPostScriptName:string;
    FTrademarkInfo :string;
    FInclCopyright     :Boolean;
    FInclFontFamilyName:Boolean;
    FInclFontSubFamName:Boolean;
    FInclFontFullName  :Boolean;
    FInclFontVersion   :Boolean;
    FInclPostScriptName:Boolean;
    FInclTrademarkInfo :Boolean;
  protected
    { Protected declarations }
  public
    { Public declarations }
    function ReadTtfFile(const TtfFileName:string):Boolean;
    property TtfError : TTtfError read FTtfError;
    property TtfPlatformID :Word read FPlatformID;
    property TtfPSpecEncID :Word read FPSpecEncID;
    property TtfLanguageID :Word read FLanguageID;
    property TtfCopyright     :string read FCopyright;
    property TtfFontFamilyName:string read FFontFamilyName;
    property TtfFontSubFamName:string read FFontSubFamName;
    property TtfFontFullName  :string read FFontFullName;
    property TtfFontVersion   :string read FFontVersion;
    property TtfPostScriptName:string read FPostScriptName ;
    property TtfTrademarkInfo :string read FTrademarkInfo;
    constructor Create(AOwner:TComponent);override;
  published
    { Published declarations }
    property InclCopyright     :Boolean read FInclCopyright write FInclCopyright default False;
    property InclFontFamilyName:Boolean read FInclFontFamilyName write FInclFontFamilyName default True;
    property InclFontSubFamName:Boolean read FInclFontSubFamName write FInclFontSubFamName default True;
    property InclFontFullName  :Boolean read FInclFontFullName write FInclFontFullName default True;
    property InclFontVersion   :Boolean read FInclFontVersion write FInclFontVersion default False;
    property InclPostScriptName:Boolean read FInclPostScriptName write FInclPostScriptName default False;
    property InclTrademarkInfo :Boolean read FInclTrademarkInfo write FInclTrademarkInfo default False;
  end;

procedure Register;

implementation
////////////////////////////////
constructor TTtfComp.Create(AOwner:TComponent);
begin
  Inherited;
  FInclCopyright:=False;
  FInclFontFamilyName:=True;
  FInclFontSubFamName:=True;
  FInclFontFullName:=True;
  FInclFontVersion:=False;
  FInclPostScriptName:=False;
  FInclTrademarkInfo:=False;
end; { constructor TTtfComp.Create }
///////////////////////////
function TTtfComp.ReadTtfFile(const TtfFileName:string):Boolean;
var
  hTtfFile:hFile;
  NamingTableAddr  :Integer;
  NameRecordsCount :Word;
  StrgsStorageAddr :Word;
  NameID,
  StringLength,
  StringOffSet:Word;
  { for reading file : Number of bytes which were red }
  GotBytes:cardinal;
  { common }
  LNumBuf:array[1..12]of byte;
  LNumBufSpec:array[1..12]of byte;
  LBuffer:array[1..4]of byte;
  LAddress:Integer absolute LBuffer;
  i:Integer;
  { for checking }
  LStrBuf:array[1..4]of char;
  { for reading NameRecords }
  LNameBuf:array[1..300]of char;
  X:Word absolute LBuffer;
  C:char;
  k:Integer;
  W:array[1..150]of widechar absolute LNameBuf;
  CurrentPos:Integer;
begin
  Result:=False;
  FTtfError:=TtfUnknownError;
  FCopyright     :='';
  FFontFamilyName:='';
  FFontSubFamName:='';
  FFontFullName  :='';
  FFontVersion   :='';
  FPostScriptName:='';
  FTrademarkInfo :='';
  { Try to open the given file }
  try
  hTtfFile:=CreateFile(PChar(TtfFileName),
                       GENERIC_READ,
                       FILE_SHARE_READ,
                       nil,
                       OPEN_EXISTING,
                       FILE_ATTRIBUTE_COMPRESSED or FILE_ATTRIBUTE_HIDDEN or
                       FILE_ATTRIBUTE_READONLY or FILE_ATTRIBUTE_SYSTEM	,
                       0);
  If hTtfFile=INVALID_HANDLE_VALUE then
  { Set ErrorCode and leave out }
  begin
    FTtfError:=TtfFileNoAccess;
    Exit
  end;

  { the file was opened so check it }
  NamingTableAddr:=0;
  SetFilePointer(hTtfFile,12,nil,FILE_BEGIN);
  For i:=1 to 24 do
  begin
    ReadFile(hTtfFile,LStrBuf,4,GotBytes,nil);
    If GotBytes=0 then
      Break
    Else
      If LStrBuf='name' then
      begin
        SetFilePointer(hTtfFile,4,nil,FILE_CURRENT);
        ReadFile(hTtfFile,LNumBuf,4,GotBytes,nil);
        LBuffer[1]:=LNumBuf[4];
        LBuffer[2]:=LNumBuf[3];
        LBuffer[3]:=LNumBuf[2];
        LBuffer[4]:=LNumBuf[1];
        NamingTableAddr:=LAddress;
        Break;
      end;
    SetFilePointer(hTtfFile,12,nil,FILE_CURRENT);
  end;
  { if NamingTableAddr wasn't set then the file
    isn't ttf so we set ErrorCode and leave out }
  If NamingTableAddr=0 then
  begin
    FTtfError:=TtfFileInvalid;
    Exit
  end;

  { NamingTableAddr was set so work further }
  { Get number of NameRecords and  Offset to
    start of string storage (from start of table)}
  NameRecordsCount:=0;
  StrgsStorageAddr:=0;
  SetFilePointer(hTtfFile,NamingTableAddr+2,nil,FILE_BEGIN);
  ReadFile(hTtfFile,LNumBuf,4,GotBytes,nil);
  LBuffer[1]:=LNumBuf[4];
  LBuffer[2]:=LNumBuf[3];
  LBuffer[3]:=LNumBuf[2];
  LBuffer[4]:=LNumBuf[1];
  NameRecordsCount:=HiWord(LAddress);
  StrgsStorageAddr:=LoWord(LAddress);
  { if NameRecordsCount and StrgsStorageAddr were not
    set then we must leave out with some ErrorCode   }
  If (NameRecordsCount=0)or(StrgsStorageAddr=0)then
  begin
    FTtfError:=TtfFileInvalid;
    Exit
  end;

  { good so far so try to read NameRecords and
    find out some information about this font }
  For i:=1 to NameRecordsCount do
  begin
    ReadFile(hTtfFile,LNumBufSpec,12,GotBytes,nil);
    LBuffer[1] :=LNumBufSpec[2];
    LBuffer[2] :=LNumBufSpec[1];
    FPlatformID :=X;

    LBuffer[1] :=LNumBufSpec[4];
    LBuffer[2] :=LNumBufSpec[3];
    FPSpecEncID :=X;

    LBuffer[1] :=LNumBufSpec[6];
    LBuffer[2] :=LNumBufSpec[5];
    FLanguageID :=X;

    LBuffer[1] :=LNumBufSpec[8];
    LBuffer[2] :=LNumBufSpec[7];
    NameID :=X;
    { retrieving Copyright notice }
    If (NameID=0)and(FPlatformID=3)and FInclCopyright then
    begin
      CurrentPos:=SetFilePointer(hTtfFile,0,nil,FILE_CURRENT);
      LBuffer[1] :=LNumBufSpec[10];
      LBuffer[2] :=LNumBufSpec[ 9];
      StringLength:=X;
      LBuffer[1]:=LNumBufSpec[12];
      LBuffer[2]:=LNumBufSpec[11];
      StringOffSet:=X;
      SetFilePointer(hTtfFile,
                     NamingTableAddr+StrgsStorageAddr+StringOffSet,
                     nil,FILE_BEGIN);
      ReadFile(hTtfFile,LNameBuf,StringLength,GotBytes,nil);
      For k:=1 to StringLength do
      if Odd(k) then
      begin
        C:=LNameBuf[k+1];
        LNameBuf[k+1]:=LNameBuf[k];
        LNameBuf[k]:=C;
      end;
      LNameBuf[StringLength+1]:=#0;
      LNameBuf[StringLength+2]:=#0;
      FCopyright:=W;
      FCopyright:=PChar(FCopyright);
      SetFilePointer(hTtfFile,CurrentPos,nil,FILE_BEGIN);
    end;
    { retrieving Font Family name }
    If (NameID=1)and(FPlatformID=3)and FInclFontFamilyName then
    begin
      CurrentPos:=SetFilePointer(hTtfFile,0,nil,FILE_CURRENT);
      LBuffer[1] :=LNumBufSpec[10];
      LBuffer[2] :=LNumBufSpec[ 9];
      StringLength:=X;
      LBuffer[1]:=LNumBufSpec[12];
      LBuffer[2]:=LNumBufSpec[11];
      StringOffSet:=X;
      SetFilePointer(hTtfFile,
                     NamingTableAddr+StrgsStorageAddr+StringOffSet,
                     nil,FILE_BEGIN);
      ReadFile(hTtfFile,LNameBuf,StringLength,GotBytes,nil);
      For k:=1 to StringLength do
      if Odd(k) then
      begin
        C:=LNameBuf[k+1];
        LNameBuf[k+1]:=LNameBuf[k];
        LNameBuf[k]:=C;
      end;
      LNameBuf[StringLength+1]:=#0;
      LNameBuf[StringLength+2]:=#0;
      FFontFamilyName:=W;
      FFontFamilyName:=PChar(FFontFamilyName);
      SetFilePointer(hTtfFile,CurrentPos,nil,FILE_BEGIN);
    end;
    { retrieving Font Subfamily name }
    If (NameID=2)and(FPlatformID=3)and FInclFontSubFamName then
    begin
      CurrentPos:=SetFilePointer(hTtfFile,0,nil,FILE_CURRENT);
      LBuffer[1] :=LNumBufSpec[10];
      LBuffer[2] :=LNumBufSpec[ 9];
      StringLength:=X;
      LBuffer[1]:=LNumBufSpec[12];
      LBuffer[2]:=LNumBufSpec[11];
      StringOffSet:=X;
      SetFilePointer(hTtfFile,
                     NamingTableAddr+StrgsStorageAddr+StringOffSet,
                     nil,FILE_BEGIN);
      ReadFile(hTtfFile,LNameBuf,StringLength,GotBytes,nil);
      For k:=1 to StringLength do
      if Odd(k) then
      begin
        C:=LNameBuf[k+1];
        LNameBuf[k+1]:=LNameBuf[k];
        LNameBuf[k]:=C;
      end;
      LNameBuf[StringLength+1]:=#0;
      LNameBuf[StringLength+2]:=#0;
      FFontSubFamName:=W ;
      FFontSubFamName:=PChar(FFontSubFamName);
      SetFilePointer(hTtfFile,CurrentPos,nil,FILE_BEGIN);
    end;
    { retrieving Full font name }
    If (NameID=4)and(FPlatformID=3)and FInclFontFullName then
    begin
      CurrentPos:=SetFilePointer(hTtfFile,0,nil,FILE_CURRENT);
      LBuffer[1] :=LNumBufSpec[10];
      LBuffer[2] :=LNumBufSpec[ 9];
      StringLength:=X;
      LBuffer[1]:=LNumBufSpec[12];
      LBuffer[2]:=LNumBufSpec[11];
      StringOffSet:=X;
      SetFilePointer(hTtfFile,
                     NamingTableAddr+StrgsStorageAddr+StringOffSet,
                     nil,FILE_BEGIN);
      ReadFile(hTtfFile,LNameBuf,StringLength,GotBytes,nil);
      For k:=1 to StringLength do
      if Odd(k) then
      begin
        C:=LNameBuf[k+1];
        LNameBuf[k+1]:=LNameBuf[k];
        LNameBuf[k]:=C;
      end;
      LNameBuf[StringLength+1]:=#0;
      LNameBuf[StringLength+2]:=#0;
      FFontFullName:=W;
      FFontFullName:=PChar(FFontFullName);
      SetFilePointer(hTtfFile,CurrentPos,nil,FILE_BEGIN);
    end;
    { retrieving Version string. In n.nn format }
    If (NameID=5)and(FPlatformID=3)and FInclFontVersion then
    begin
      CurrentPos:=SetFilePointer(hTtfFile,0,nil,FILE_CURRENT);
      LBuffer[1] :=LNumBufSpec[10];
      LBuffer[2] :=LNumBufSpec[ 9];
      StringLength:=X;
      LBuffer[1]:=LNumBufSpec[12];
      LBuffer[2]:=LNumBufSpec[11];
      StringOffSet:=X;
      SetFilePointer(hTtfFile,
                     NamingTableAddr+StrgsStorageAddr+StringOffSet,
                     nil,FILE_BEGIN);
      ReadFile(hTtfFile,LNameBuf,StringLength,GotBytes,nil);
      For k:=1 to StringLength do
      if Odd(k) then
      begin
        C:=LNameBuf[k+1];
        LNameBuf[k+1]:=LNameBuf[k];
        LNameBuf[k]:=C;
      end;
      LNameBuf[StringLength+1]:=#0;
      LNameBuf[StringLength+2]:=#0;
      FFontVersion :=W;
      FFontVersion:=PChar(FFontVersion);
      SetFilePointer(hTtfFile,CurrentPos,nil,FILE_BEGIN);
    end;
    { retrieving Postscript name for the font }
    If (NameID=6)and(FPlatformID=3)and FInclPostScriptName then
    begin
      CurrentPos:=SetFilePointer(hTtfFile,0,nil,FILE_CURRENT);
      LBuffer[1] :=LNumBufSpec[10];
      LBuffer[2] :=LNumBufSpec[ 9];
      StringLength:=X;
      LBuffer[1]:=LNumBufSpec[12];
      LBuffer[2]:=LNumBufSpec[11];
      StringOffSet:=X;
      SetFilePointer(hTtfFile,
                     NamingTableAddr+StrgsStorageAddr+StringOffSet,
                     nil,FILE_BEGIN);
      ReadFile(hTtfFile,LNameBuf,StringLength,GotBytes,nil);
      For k:=1 to StringLength do
      if Odd(k) then
      begin
        C:=LNameBuf[k+1];
        LNameBuf[k+1]:=LNameBuf[k];
        LNameBuf[k]:=C;
      end;
      LNameBuf[StringLength+1]:=#0;
      LNameBuf[StringLength+2]:=#0;
      FPostScriptName:=W;
      FPostScriptName:=PChar(FPostScriptName);
      SetFilePointer(hTtfFile,CurrentPos,nil,FILE_BEGIN);
    end;
    { retrieving trademark notice/information for this font }
    If (NameID=7)and(FPlatformID=3)and FInclTrademarkInfo then
    begin
      CurrentPos:=SetFilePointer(hTtfFile,0,nil,FILE_CURRENT);
      LBuffer[1] :=LNumBufSpec[10];
      LBuffer[2] :=LNumBufSpec[ 9];
      StringLength:=X;
      LBuffer[1]:=LNumBufSpec[12];
      LBuffer[2]:=LNumBufSpec[11];
      StringOffSet:=X;
      SetFilePointer(hTtfFile,
                     NamingTableAddr+StrgsStorageAddr+StringOffSet,
                     nil,FILE_BEGIN);
      ReadFile(hTtfFile,LNameBuf,StringLength,GotBytes,nil);
      For k:=1 to StringLength do
      if Odd(k) then
      begin
        C:=LNameBuf[k+1];
        LNameBuf[k+1]:=LNameBuf[k];
        LNameBuf[k]:=C;
      end;
      LNameBuf[StringLength+1]:=#0;
      LNameBuf[StringLength+2]:=#0;
      FTrademarkInfo:=W;
      FTrademarkInfo:=PChar(FTrademarkInfo);
    end;
  end;
  Result:=True;
  finally
    If Result then
    { our work is done }
      FTtfError:=TtfFileNoError;
    CloseHandle(hTtfFile)
  end
end; { }
//////////////////////////////
procedure Register;
begin
  RegisterComponents('RAS', [TTtfComp]);
end;

end.
 